package Pronto::CellRendererPixText;

use strict;

use Glib qw(G_PARAM_READWRITE);
use Gtk2;
use Glib::Object::Subclass
	"Gtk2::CellRenderer",
	properties => [
	  Glib::ParamSpec->object ('pixbuf',
	                           'Pixbuf',
	                           'The pixbuf to display',
				   'Gtk2::Gdk::Pixbuf', G_PARAM_READWRITE),
	  Glib::ParamSpec->string ('text',
	                           'Text',
	                           'The text string to display',
				   "", G_PARAM_READWRITE),
	  Glib::ParamSpec->boolean('weight_set',
	                           'Weight set',
	                           'Should the text be printed in bold?',
				   0, G_PARAM_READWRITE),
	  Glib::ParamSpec->string ('font-string',
	                           'Font String',
	                           'The font used',
				   'Fixedsys 12', G_PARAM_READWRITE),
	  Glib::ParamSpec->string ('weight',
	                           'Weight',
	                           'The weight of the font',
				   'bold', G_PARAM_READWRITE),
	],
  ;


sub INIT_INSTANCE {
	my $self = shift;
}

sub setup_text {
	my ($self, $widget) = @_;
	my $font = Gtk2::Pango::FontDescription->from_string($self->{font_string});
#	$font->set_weight($self->{'weight'}) if ($self->{'weight'} and $self->{'weight_set'});
	$font->set_weight('bold') if ($self->{'weight'} and $self->{'weight_set'});
	my $layout = $widget->create_pango_layout($self->{'text'});
	$layout->set_font_description($font);

	return ($layout->get_pixel_size, $layout);
}

sub GET_SIZE {
	my ($cell, $widget, $cell_area) = @_;
	my ($x_offset, $y_offset) = (0, 0);
	
	my ($width, $height) = $cell->setup_text($widget);

	if ($cell->{pixbuf}) {
		my $pw = $cell->{pixbuf}->get_width;
		my $ph = $cell->{pixbuf}->get_height;
		$width += $pw;
		$height = $ph if $ph > $height;
	}

	return ($x_offset, $y_offset, $width, $height);
}

sub RENDER {
	my ($cell, $window, $widget, $background_area, $cell_area, $expose_area, $flags) = @_;
	my $pix_width  = $cell->get('pixbuf')->get_width;
	my $pix_height = $cell->get('pixbuf')->get_height;
	my $state;
	
	if ($flags & 'selected') {
		$state = $widget->has_focus() ? 'selected' : 'active';
	} else {
		$state = $widget->state() eq 'insensitive' ? 'insensitive' : 'normal';
	}

	if ($pix_width > 1) {
		$window->draw_pixbuf(
				$widget->get_style->fg_gc($widget->state),
				$cell->get("pixbuf"),
				0,
				0,
				$cell_area->x(),
				$cell_area->y() + int(($cell_area->height - $pix_height)/2),
				$pix_width,
				$pix_height,
				"GDK_RGB_DITHER_NONE",
				0,
				0
		);
	} else {
		$pix_width=0;
	}

	my ($width, $height, $layout) = $cell->setup_text($widget);

	$widget->get_style->paint_layout($window,
			$state,
			1,
			$cell_area,
			$widget,
			"cellrenderertext",
			$cell_area->x + $pix_width + 1,
			$cell_area->y + ($cell_area->height - $height) / 2,
			$layout
	);
}

1;
