# $Id: FolderTree.pm,v 1.10 2002/05/14 08:36:11 muhri Exp $
# -*- perl -*-
package Pronto::FolderTree;
use strict;
use vars qw( @new_count @msg_count %parent );

use SelfLoader;

use constant FALSE => 0;
use constant TRUE => 1;

use constant FOLDNAM_COL => 0;
use constant MSG_COL     => 1;
use constant NEWMSG_COL  => 2;
use constant BOLD_COL    => 3;
use constant ICON_COL    => 4;

use Carp;

sub insert_a_folder_node {
	my (@row) = @_; 

	# $row[0]: foldername
	# $row[1]: id
	# $row[2]: sibling
	# $row[3]: parent
	
        $main::foldernametoid{$row[0]} = $row[1];
        $main::folderidtoname{$row[1]} = $row[0];
        $parent{$row[1]} = $row[3];
        my $title = [ $row[0] , "" , "" ];

	my $model = $main::folder_tree->get_model;
	my $iter;

	if (($row[2] == 65535) and (not defined $row[3] or $row[3] == -1)) {
		# Last node in current level and no parent
		$iter = $model->append(undef);
		set_folder_name($model, $iter, $row[0]);
		$model->set($iter, MSG_COL, 0, NEWMSG_COL, 0, BOLD_COL, FALSE);
		$main::ft_node[$row[1]] = $iter;
        } elsif ($row[2] == 65535) {
		# Last node in current level
		$iter = $model->append($main::ft_node[$row[3]]);
		set_folder_name($model, $iter, $row[0]);
		$model->set($iter, MSG_COL, 0, NEWMSG_COL, 0, BOLD_COL, FALSE);
		$main::ft_node[$row[1]] = $iter;
	} elsif (not defined $row[3] or $row[3] == -1) {
		# A node with no parent
		print "Help! Unhandeled folder tree!\n";
        } else {
		# All the other nodes
		$iter = $model->insert_before($main::ft_node[$row[3]], $main::ft_node[$row[2]]);
		set_folder_name($model, $iter, $row[0]);
		$model->set($iter, MSG_COL, 0, NEWMSG_COL, 0, BOLD_COL, FALSE);
		$main::ft_node[$row[1]] = $iter;
        }
}

sub insert_folder_nodes {
	my($parent,$sibling)=@_;
	my($sql,$query,@row);
	if(not defined $sibling) {
		$sibling = 65535;
	}
	if(not defined $parent) {
		$sql = "select name, id, sibling, parent from boxlist where ((sibling=?) and ((parent=-1) or (parent is NULL)))";
		$query = $main::conn->prepare($sql);
		$query->execute($sibling);
	}
	else {
		$sql = "select name, id, sibling, parent from boxlist where ((sibling=?) and (parent=?))";
		$query = $main::conn->prepare($sql);
		$query->execute($sibling,$parent);
	}
	if(@row=$query->fetchrow_array()) {
		&insert_a_folder_node(@row);
		&insert_folder_nodes($row[1]);
		if(not defined $parent) {
			&insert_folder_nodes(undef,$row[1]);
		}
		else {
			&insert_folder_nodes($parent,$row[1]);
		}
	}
	&set_folder_style();
	$main::folder_tree->expand_all;
}

sub pack_box_onto_column_widget
{
	my ($hbox, $button, $label);
	$hbox = new Gtk2::HBox(0,0);
	$button = new Gtk2::Button("^");
	$button->set_size_request(10,10);
#	$button->signal_connect("clicked", 
#		sub {
#			if ($Pronto::MainWindow::FTDetach->sensitive()) {
#				&Pronto::MainWindow::DetachFolderTree();
#			} else {
#				&Pronto::MainWindow::ReAttachFolderTree();
#     			}
#		}
#		);	
	$label = new Gtk2::Label(_("New"));
	$hbox->pack_start($label,0,0,0);
	$hbox->pack_start($button,0,0,0);
	$hbox->show_all();
#	$main::folder_tree->set_column_widget(2,$hbox);
	return;
}	


sub fix_boxlist
{
	my($parent) = @_;
	my($sql,$query,$query2,@row,$lastid);
	if(defined $parent) {
		$sql = "select id,sibling,parent from boxlist where parent=? order by id desc";
		$query=$main::conn->prepare($sql);
		$query->execute($parent);
	}
	else {
		$sql = "select id,sibling,parent from boxlist where (parent=-1 or parent is NULL) order by id desc";
		$query=$main::conn->prepare($sql);
		$query->execute();
	}
	$lastid=65535;
	while(@row=$query->fetchrow_array()) {
		$sql = "update boxlist set sibling=? where id=?";
		$query2=$main::conn->prepare($sql);
		$query2->execute($lastid,$row[0]);
		$lastid=$row[0];
		&fix_boxlist($row[0]);
	}
}



sub init_folder_tree {
 	my ($sql, $query, @row, $column, $model, $renderer);
        my @titles = ( _("Folder"), _("Msg"), _("New") );
	
 	$model = Gtk2::TreeStore->new('Glib::String', 'Glib::String', 'Glib::String', 'Glib::Boolean', 'Gtk2::Gdk::Pixbuf');
	$main::folder_tree = Gtk2::TreeView->new($model);
	$main::folder_tree->{'no_dragging'} = 1;
 	&pack_box_onto_column_widget();

	# Set up the column for the foldernames with a possible pixbuf for a vfolder symbol
	$renderer = Pronto::CellRendererPixText->new;
	$renderer->set('font-string' => $main::prefs{'FontMLNormal'});
	$renderer->set('weight' => $main::bold);
	$column = Gtk2::TreeViewColumn->new_with_attributes($titles[0], $renderer, 'text' => FOLDNAM_COL, 'weight_set' => BOLD_COL, 'pixbuf' => ICON_COL);
	$column->set_resizable(1);
	$column->set_sizing('fixed');
	if (not defined $main::prefs{'FolderTreeColWidth0'}) {
		$column->set_fixed_width(130);
	} else {
		$column->set_fixed_width($main::prefs{'FolderTreeColWidth0'});
	}
	$main::folder_tree->append_column($column);
	$main::folder_tree->{'pix'}->{'vfolder'} = &main::new_pixbuf('vfolder.xpm');
	$main::folder_tree->{'pix'}->{'search'} = &main::new_pixbuf('search.xpm');
	$main::folder_tree->{'pix'}->{'dummy'} = Gtk2::Gdk::Pixbuf->new('GDK_COLORSPACE_RGB', 1, 8, 1, 1);

	# Set up the other columns
	if($main::prefs{'msgcolumn'} eq "y") { 
		$renderer = Gtk2::CellRendererText->new;
		$renderer->set('font-desc' => Gtk2::Pango::FontDescription->from_string($main::prefs{'FontMLNormal'}));
		$renderer->set('weight' => $main::bold);
		$column = Gtk2::TreeViewColumn->new_with_attributes($titles[1], $renderer, 'text' => MSG_COL, 'weight_set' => BOLD_COL);
		$column->set_resizable(1);
		$column->set_sizing('fixed');
 		if (not defined $main::prefs{'FolderTreeColWidth1'}) {
			$column->set_fixed_width(30);
		} else {
			$column->set_fixed_width($main::prefs{'FolderTreeColWidth1'});
		}
		$main::folder_tree->append_column($column);
 	}
 	if($main::prefs{'newcolumn'} eq "y") { 
		$renderer = Gtk2::CellRendererText->new;
		$renderer->set('font-desc' => Gtk2::Pango::FontDescription->from_string($main::prefs{'FontMLNormal'}));
		$renderer->set('weight' => $main::bold);
		$column = Gtk2::TreeViewColumn->new_with_attributes($titles[2], $renderer, 'text' => NEWMSG_COL, 'weight_set' => BOLD_COL);
		$column->set_resizable(1);
		$column->set_sizing('fixed');
 		if (not defined $main::prefs{'FolderTreeColWidth2'}) {
			$column->set_fixed_width(20);
		} else {
			$column->set_fixed_width($main::prefs{'FolderTreeColWidth2'});
		}
		$main::folder_tree->append_column($column);
 	}

	my $selection = $main::folder_tree->get_selection;
 	$selection->signal_connect("changed", sub { 
					return 1 if ($main::dragging);
				
	 				if ($main::prefs{'messageview'} eq "clist") {
	 					&Pronto::MessageList::refresh_messages;
	 				} else { 
						&Pronto::MessageList::refresh_ctree("switch") 
					}
	 
	 				if($main::prefs{'onfolderchange'} eq 'top') {
						&Pronto::MessageList::goto_top;
	 				}
	 				elsif($main::prefs{'onfolderchange'} eq 'next unread') {
						&Pronto::MessageList::goto_next_unread_msg;
	 				}
#	 				&Pronto::Read::clear_widget; #Is this still needed? (Florian)
	 				});
		 
#       	$main::folder_tree->signal_connect("unselect_row", sub { 
#		$main::message_clist->get_model->clear; 
#		if ($main::root_window->{'button'}->visible) { 
#			foreach(keys(%main::folderidtoname)) {
#			if ($_ >= 1000) {
#				$main::folder_tree->get_model->set($main::ft_node[$_], MSG_COL, "");
#				$main::folder_tree->get_model->set($main::ft_node[$_], NEWMSG_COL, "");
#			}
#		}
#		$main::root_window->{'button'}->hide;}
#		&Pronto::Read::clear_widget;
#	});
	
	if (!$main::prefs{'Drag'}) {
		$main::folder_tree->set_reorderable(1);
	 	$main::folder_tree->get_model->signal_connect("row-inserted", \&folder_moved);
	} else {
		&main::set_folder_tree_drag_signals();
	}			
	
#	$main::folder_tree->signal_connect("resize-column" => sub {my ($d, $c, $w)=@_; $main::prefs{'FolderTreeColWidth'. $c}=$w;});
 	%main::foldernametoid = ();
 	%main::folderidtoname = ();
	
	&insert_folder_nodes();
	#check that the boxlist table is correct, and all folders were inserted
	$sql = "select name,id,sibling,parent from boxlist order by parent,sibling";
 	$query = $main::conn->prepare($sql);
        $query->execute();
        while(@row=$query->fetchrow_array()) {
		if(not defined $main::ft_node[$row[1]]) {
			print _("oops, $row[0] folder was missed, make sure boxlist table is correct\n");
			$main::folder_tree->destroy();
			&fix_boxlist();
			&init_folder_tree();			
##			&insert_a_folder_node(@row);
		}
	}

	$main::folder_tree->signal_connect('button_press_event', \&rightclick);
 	$main::folder_tree->{'popup'} =  new Gtk2::Menu;
 	$main::folder_tree->{'popup'}->{'sep'} = new Gtk2::MenuItem;
 	$main::folder_tree->{'popup'}->{'sep'}->show;
# 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'sep'});
 	$main::folder_tree->{'popup'}->{'new'} = new Gtk2::MenuItem(_("New Folder"));
 	$main::folder_tree->{'popup'}->{'new'}->signal_connect("activate" => \&Pronto::FolderTree::new_folder);
 	$main::folder_tree->{'popup'}->{'new'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'new'});
 	$main::folder_tree->{'popup'}->{'refilter'} = new Gtk2::MenuItem(_("Refilter Folder"));
 	$main::folder_tree->{'popup'}->{'refilter'}->signal_connect("activate" => \&Pronto::FolderTree::refilter_folder);
 	$main::folder_tree->{'popup'}->{'refilter'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'refilter'});
 	$main::folder_tree->{'popup'}->{'delete'} = new Gtk2::MenuItem(_("Delete Folder"));
 	$main::folder_tree->{'popup'}->{'delete'}->signal_connect("activate" => \&Pronto::FolderTree::delete_folder);
 	$main::folder_tree->{'popup'}->{'delete'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'delete'});
 	$main::folder_tree->{'popup'}->{'rename'} = new Gtk2::MenuItem(_("Rename Folder"));
 	$main::folder_tree->{'popup'}->{'rename'}->signal_connect("activate" => \&Pronto::FolderTree::rename_folder);
 	$main::folder_tree->{'popup'}->{'rename'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'rename'});
 	$main::folder_tree->{'popup'}->{'sep2'} = new Gtk2::MenuItem;
 	$main::folder_tree->{'popup'}->{'sep2'}->show;
	$main::folder_tree->{'popup'}->{'markasread'} = new Gtk2::MenuItem(_("Mark as Read"));
	$main::folder_tree->{'popup'}->{'markasread'}->show();
	$main::folder_tree->{'popup'}->{'markasread'}->signal_connect("activate" => \&mark_folder_as_read);
 	$main::folder_tree->{'popup'}->{'new_view'} = new Gtk2::MenuItem(_("Open in a new View"));
	$main::folder_tree->{'popup'}->{'new_view'}->show();
#	$main::folder_tree->{'popup'}->{'new_view'}->signal_connect("activate", sub { &Pronto::CListView::new() });
	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'sep2'});
	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'markasread'});
	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'new_view'});
	$main::folder_tree->{'popup'}->{'sep3'} = new Gtk2::MenuItem();
	$main::folder_tree->{'popup'}->{'sep3'}->show();
	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'sep3'});	
	$main::folder_tree->{'popup'}->{'import'} = new Gtk2::MenuItem(_("Import into Folder"));
 	$main::folder_tree->{'popup'}->{'import'}->signal_connect("activate" => \&Pronto::FolderTree::init_import_folder);
 	$main::folder_tree->{'popup'}->{'import'}->show;
       	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'import'});
 	$main::folder_tree->{'popup'}->{'export'} = new Gtk2::MenuItem(_("Export Folder"));
 	$main::folder_tree->{'popup'}->{'export'}->signal_connect("activate" => \&Pronto::FolderTree::init_export_folder);
 	$main::folder_tree->{'popup'}->{'export'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'export'});
 	$main::folder_tree->{'popup'}->{'sep3'} = new Gtk2::MenuItem;
 	$main::folder_tree->{'popup'}->{'sep3'}->show;    
       	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'sep3'});
 	$main::folder_tree->{'popup'}->{'emptydeleted'} = new Gtk2::MenuItem(_("Empty Deleted Items"));
	$main::folder_tree->{'popup'}->{'emptydeleted'}->signal_connect("activate"=>\&Pronto::FolderTree::empty_trash);
 	$main::folder_tree->{'popup'}->{'emptydeleted'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'emptydeleted'});
 	$main::folder_tree->{'popup'}->{'emptyfolder'} = new Gtk2::MenuItem(_("Empty Selected folder"));
 	$main::folder_tree->{'popup'}->{'emptyfolder'}->signal_connect("activate" => \&Pronto::FolderTree::empty_folder);
 	$main::folder_tree->{'popup'}->{'emptyfolder'}->show;
 	$main::folder_tree->{'popup'}->append($main::folder_tree->{'popup'}->{'emptyfolder'});

	$main::folder_tree->{'no_dragging'} = 0;
}


sub get_folder_id
{
	my ($model, $selection) = $main::folder_tree->get_selection->get_selected;
	if (!$selection) { return undef; }
	my $foldername = get_folder_name($model, $selection);
	my $id = $main::foldernametoid{$foldername};
	return $id;
}

sub get_folder_name
{
	my ($model, $iter) = @_;
	if (!$iter) { return undef; }
	
	my $foldername = $model->get($iter, FOLDNAM_COL);
	my $newmsgs    = $model->get($iter, NEWMSG_COL);
	
 	if($main::prefs{'newcolumn'} eq "n" && $main::prefs{'msgcolumn'} eq "n" && $newmsgs > 0) { 
		$foldername =~ s/(.*) \($newmsgs\)$/$1/g;
	}

	return $foldername;
}

sub set_folder_name
{
	my ($model, $iter, $name, $newmsgs) = @_;
	if (!$iter) { return undef; }

	$newmsgs = $model->get($iter, NEWMSG_COL) if (not defined $newmsgs);
 	if($newmsgs && $main::prefs{'newcolumn'} eq "n" && $main::prefs{'msgcolumn'} eq "n" && $newmsgs > 0) { 
		$name .= " ($newmsgs)";
	}

	$model->set($iter, FOLDNAM_COL, $name);
}

sub refresh_folder_tree {
	my ($play) = @_;
	my ($sql, $query, @row, $style);
	my $treestore   = $main::folder_tree->get_model;
	my $oldnewcount = $treestore->get($main::ft_node[0], NEWMSG_COL);
	@new_count = ();
	@msg_count = ();

	$msg_count[0] = $new_count[0] = 0;

	if ($main::prefs{'DatabaseDriver'} eq 'mysql') {
		# This should be quicker for mysql databases
		$sql = "select boxid, newmsg, count(*) from messages group by boxid, newmsg";
		$query = $main::conn->prepare($sql);
		$query->execute();
		while (@row=$query->fetchrow_array()) {
			$msg_count[0] += $row[2];
			$msg_count[$row[0]] += $row[2];

			if ($row[1] and $row[1] eq 'y') {
				$new_count[0] += $row[2];
				$new_count[$row[0]] = $row[2];
			} else {
				$new_count[$row[0]] = 0;
			}
		}
	}
	else
	{
		$sql = "select newmsg, boxid from messages";
		$query = $main::conn->prepare($sql);
		$query->execute();
		while (@row=$query->fetchrow_array()) {
			if (($row[0]) and ($row[0] eq "y")) {
				$new_count[$row[1]]++;
				$new_count[0]++;
			}
			$msg_count[$row[1]]++;
			$msg_count[0]++;
		}
	}
	
	if($main::prefs{'RecMsgCount'} eq 'y') {
		$new_count[0]=0;
		$msg_count[0]=0;
		&main::message_count(0,\@msg_count,\@new_count);
	}

	foreach (keys(%main::folderidtoname)) {
		if ($_ >= 1000) { next }
		if (!$msg_count[$_]) { $msg_count[$_] = 0;}
		if (!$new_count[$_]) { $new_count[$_] = 0;}
		$treestore->set($main::ft_node[$_], MSG_COL, $msg_count[$_]);
		$treestore->set($main::ft_node[$_], NEWMSG_COL, $new_count[$_]);
		if ($new_count[$_] != 0 or ($msg_count[$_] != 0 and $_ == 2)) {
			if ($main::prefs{'FolderNewBold'} eq "y" && $_ != 2) {
				$treestore->set($main::ft_node[$_], BOLD_COL, TRUE);
			}
			if ($main::prefs{'newcolumn'} eq 'n' and $main::prefs{'msgcolumn'} eq 'y') {
				my $temp = $treestore->get($main::ft_node[$_], MSG_COL);
				$temp =~ s/\(*\)//g;
				$treestore->set($main::ft_node[$_], MSG_COL, "$temp\($new_count[$_]\)");
			}
			if (($main::prefs{'newcolumn'} eq 'n' and $main::prefs{'msgcolumn'} eq 'n') 
			    or (defined $play && $play == 2)) {
				if($_ == 2) {
					set_folder_name($treestore, $main::ft_node[$_], $main::folderidtoname{$_}, $msg_count[$_]);
				} else {
					set_folder_name($treestore, $main::ft_node[$_], $main::folderidtoname{$_});
				}
			}

		} else {
			$treestore->set($main::ft_node[$_], BOLD_COL, FALSE);

			my $temp = $treestore->get($main::ft_node[$_], MSG_COL);
			$temp =~ s/\(*\)//g;
			$treestore->set($main::ft_node[$_], MSG_COL, $temp);
			set_folder_name($treestore, $main::ft_node[$_], $main::folderidtoname{$_});
		}
	}

	my $newcount = $treestore->get($main::ft_node[0], NEWMSG_COL);
	if ($oldnewcount ne "" && defined $play && $play == 1) {
		if (($main::prefs{'PlaySound'} eq "y") && ($newcount > $oldnewcount)) {
			system("$main::prefs{'SndCmd'} $main::prefs{'SndFl'}");
		}	
	}
	&main::set_win_title;
	return 1;
}

sub numerically { $a <=> $b; }

1;

__DATA__

sub new_folder {
  	
	my ($virtual,$querycontent,$body) = @_;
  	my ($fldr_dlg, $fd_hbox, $fd_label, $fd_button, $fd_ok, $fd_entry,
      	$fldr_ctree, $fldr_scroll, $fd_label2, %lparent, $selected);
	my ($model, $selection) = $main::folder_tree->get_selection->get_selected;
  	if ($selection) {
		$selected = $main::foldernametoid{get_folder_name($model, $selection)};
	}
  	$fldr_dlg = new Gtk2::Dialog;
  	$fldr_dlg->set_default_size(&main::get_win_size("fldrdlg",300,300));
	$fldr_dlg->set_resizable(1);
	$fldr_dlg->set('allow-shrink', 1);
	$fldr_dlg->set('allow-grow', 1);
  	$fldr_dlg->set_position(-mouse);
  	$fldr_dlg->signal_connect("destroy" => sub { $fldr_dlg->destroy; });
#  	$fldr_dlg->signal_connect("delete-event" => sub { Gtk2->false } );
  	$fldr_dlg->set_title(_("Enter Folder Name..."));
  	$fldr_dlg->set_border_width(0);
  	$fd_hbox=new Gtk2::HBox(0,0);
  	$fd_hbox->set_border_width(10);
  	$fldr_dlg->vbox->pack_start($fd_hbox,0,1,0);
  	$fd_hbox->show;
  	$fd_label=new Gtk2::Label _("Folder Name: ");
  	$fd_label->set_alignment(0.5,0.5);
  	$fd_hbox->pack_start($fd_label,0,1,0);
  	$fd_label->show;
  	$fd_entry=new Gtk2::Entry;
  	$fd_entry->set_text('');
  	$fd_entry->set_editable('1');
  	$fd_hbox->pack_start($fd_entry,1,1,0);
  	$fd_entry->show;
  	$fd_label2=new Gtk2::Label _("Create new folder in:");
  	$fd_label2->set_alignment(0.0,0.5);
  	$fldr_dlg->vbox->pack_start($fd_label2,0,1,0);
  	$fd_label2->show;

  	$fldr_ctree = Gtk2::TreeView->new($model);
	my $renderer = Gtk2::CellRendererText->new;
	$renderer->set('font-desc' => Gtk2::Pango::FontDescription->from_string($main::prefs{'FontMLNormal'}));
	$column = Gtk2::TreeViewColumn->new_with_attributes("Folders", $renderer, 'text' => FOLDNAM_COL);
	$column->set_resizable(0);
	$column->set_fixed_width(130);
	$fldr_ctree->append_column($column);
	$fldr_ctree->set_headers_visible(FALSE);
	$fldr_ctree->expand_all;

 	if ($selection) { $fldr_ctree->get_selection->select_iter($selection); }
		
  	$fldr_ctree->show;
  	$fldr_scroll = new Gtk2::ScrolledWindow(undef,undef);
  	$fldr_scroll->set_policy('automatic', 'automatic');
	$fldr_scroll->set_shadow_type('etched-in');
  	$fldr_scroll->show;
  	$fldr_scroll->add($fldr_ctree);
  	$fldr_dlg->vbox->pack_start($fldr_scroll,1,1,0);
  	$fd_ok = Gtk2::Button->new_from_stock('gtk-ok');
  	$fd_ok->can_default(1);
  	$fd_ok->show;
  	$fldr_dlg->action_area->pack_start($fd_ok,1,1,0);
  	$fd_button = Gtk2::Button->new_from_stock('gtk-cancel');
  	$fd_button->signal_connect("clicked", sub { $fldr_dlg->destroy; });
  	$fldr_dlg->action_area->pack_start($fd_button,1,1,0);
  	$fd_button->show;
  	$fldr_dlg->show;
	if (defined $virtual and $virtual eq "y") {
		$fd_ok->signal_connect("clicked" => \&add_new_folder, [$fd_entry,
			$fldr_dlg, $fldr_ctree, $virtual, $querycontent, $body]);
		$fd_entry->signal_connect("activate" => \&add_new_folder, [$fd_entry,
			$fldr_dlg, $fldr_ctree, $virtual, $querycontent, $body]);
	} else {
		$fd_ok->signal_connect("clicked" => \&add_new_folder, [$fd_entry,
			$fldr_dlg, $fldr_ctree, undef, undef, undef]);
		$fd_entry->signal_connect("activate" => \&add_new_folder, [$fd_entry,
			$fldr_dlg, $fldr_ctree, undef, undef, undef]);
	}
  	$fldr_dlg->signal_connect("size-request" => \&main::save_win_size, ["fldrdlg", $fldr_dlg->window]);
	$fd_entry->grab_focus();  	
	return 1;
}


sub folderlist_drag_data_received {
	my ($widget, $context, $x, $y, $data, $info, $time) = @_;
	if (($data->length >= 0) && ($data->format == 8)) {
		
		if (!@main::dragged) {
			&Pronto::Data::Message::move(get_folder_id,$data->data);
		} else {
			&Pronto::Data::Message::move(get_folder_id,@main::dragged);
		}
		
		$main::folder_tree->get_selection->select_iter($main::ft_node[$main::dragging]);
		&Pronto::MessageList::refresh();
		&Pronto::FolderTree::refresh_folder_tree();
		$context->finish (1, 0, $time);
		return 1;
	}
	$context->finish (0, 0, $time);
}

sub folderlist_drag_motion 
{
        my ($widget, $context, $x, $y, $time) = @_;
	my ($path, $position) = $widget->get_dest_row_at_pos($x, $y);
	$widget->get_selection->select_path($path);
	$context->status ($context->suggested_action, $time);
	return 1;
}	

sub update_boxlist {
	if (not defined $main::folder_tree->{'has_been_reordered'}) { return 1; }
	$main::folder_tree->{'has_been_reordered'} = undef;
	
	my $model = $main::folder_tree->get_model;
	my $path = Gtk2::TreePath->new_from_string('0:0');
	my %boxlistparent  = ();
	my %boxlistsibling = ();
	my $parentref  = \%boxlistparent;
	my $siblingref = \%boxlistsibling;

	($parentref, $siblingref) = &update_boxlist_recurse($model, $path, 0, $parentref, $siblingref);
	%boxlistparent  = %$parentref;
	%boxlistsibling = %$siblingref;

 	my $sql = "SELECT id from boxlist";
	my $query = $main::conn->prepare($sql);
	$query->execute();
	
	if ((keys %boxlistparent) == ($query->rows - 1)) {
		$sql = "UPDATE boxlist SET parent = ?, sibling = ? WHERE id = ?";
		while ( my ($id, $parent) = each %boxlistparent) {
			my $sibling = $boxlistsibling{$id};
			if ((defined $parent) && (defined $sibling)) {
				$query = $main::conn->prepare($sql);
				$query->execute($parent, $sibling, $id);
			}
		}
	} else {
		my $num = (keys %boxlistparent);
		print "Could not update foldertree! (".$num." != ".($query->rows - 1).")\n";
	}

	$Pronto::FolderMenus::mv_menu_menus{0}->detach;
	$Pronto::FolderMenus::pmv_menus{0}->detach;
	$Pronto::FolderMenus::cp_menu_menus{0}->detach;
	$Pronto::FolderMenus::pcp_menus{0}->detach;
#	$Pronto::FolderMenus::mv_menu_menus{0}->destroy;
#	$Pronto::FolderMenus::pmv_menus{0}->destroy;
#	$Pronto::FolderMenus::cp_menu_menus{0}->destroy;
#	$Pronto::FolderMenus::pcp_menus{0}->destroy;

	($Pronto::FolderMenus::mv_menu_menus{0},$Pronto::FolderMenus::pmv_menus{0},$Pronto::FolderMenus::cp_menu_menus{0},$Pronto::FolderMenus::pcp_menus{0})=&Pronto::FolderMenus::mvcp_menu(0);
	$Pronto::FolderMenus::mv_menu_items{0}->set_submenu($Pronto::FolderMenus::mv_menu_menus{0});
	$Pronto::FolderMenus::pmv_items{0}->set_submenu($Pronto::FolderMenus::pmv_menus{0});
	$Pronto::FolderMenus::cp_menu_items{0}->set_submenu($Pronto::FolderMenus::cp_menu_menus{0});
	$Pronto::FolderMenus::pcp_items{0}->set_submenu($Pronto::FolderMenus::pcp_menus{0});

	return 0;
}

sub update_boxlist_recurse {
	my $model   = shift;
	my $path    = shift;
	my $parent  = shift;
	my $parentref  = shift;
	my $siblingref = shift;
	my %boxlistparent  = %$parentref;
	my %boxlistsibling = %$siblingref;

	my $iter = $model->get_iter($path);
	my $name = $model->get($iter, FOLDNAM_COL);
	my $id   = $main::foldernametoid{$name};
	$main::ft_node[$id] = $iter;
	
	# Has the very node itself a parent?
	$path->down;
	$iter = $model->get_iter($path);
	if ((defined $iter) && ($model->iter_is_valid($iter))) {
		($parentref, $siblingref) = &update_boxlist_recurse($model, $path, 
							$id, \%boxlistparent, \%boxlistsibling);
		%boxlistparent  = %$parentref;
		%boxlistsibling = %$siblingref;
	}
	$path->up;

	$path->next;
	$iter = $model->get_iter($path);
	while ((defined $iter) && ($model->iter_is_valid($iter))) {
		$name = $model->get($iter, FOLDNAM_COL);
		$boxlistparent{$id}  = $parent;
		$boxlistsibling{$id} = $main::foldernametoid{$name};

		$path->down;
		my $testiter = $model->get_iter($path);
		if ((defined $testiter) && ($model->iter_is_valid($testiter))) {
			($parentref, $siblingref) = &update_boxlist_recurse($model, $path, 
						$boxlistsibling{$id}, \%boxlistparent, \%boxlistsibling);
			%boxlistparent  = %$parentref;
			%boxlistsibling = %$siblingref;
		}
		$path->up;
		
		$id = $main::foldernametoid{$name};
		$main::ft_node[$id] = $iter;
		$path->next;
		$iter = $model->get_iter($path);
	}
	
	# This block is for the case, in which a tree consists only of one leaf
	# but has a subtree attached. In this case, the "while" above isn't executed.
	$path->prev;
	$path->down;
	$iter = $model->get_iter($path);
	if ((defined $iter) && ($model->iter_is_valid($iter))) {
		($parentref, $siblingref) = &update_boxlist_recurse($model, $path, $id, \%boxlistparent, \%boxlistsibling);
		%boxlistparent  = %$parentref;
		%boxlistsibling = %$siblingref;
	}
	$path->up;

	$boxlistparent{$id}  = $parent;
	$boxlistsibling{$id} = 65535;

	return (\%boxlistparent, \%boxlistsibling);
}

sub folder_moved {
	if ($main::folder_tree->{'no_dragging'}) { return 1; }

	# Some kind of primitive lock mechanism
	$main::folder_tree->{'has_been_reordered'} = 1;

	# I cannot do the updates while still dragging, as the whole TreeStore
	# isn't in a well defined state. Therefore I use this hack to update
	# everything just a litte bit later.
	Glib::Timeout->add(100, \&update_boxlist);

	return 1;
}

sub rename_folder {
	my ($sql, $query, $foldername, $curfolder, @row, $win,$hbox, $ok, $cancel, $lbl, $entry, $newfoldername);
	$curfolder = get_folder_id();
	if (!$curfolder) { &main::err_dialog("Select a folder first!"); return 1; }
	$foldername = $main::folderidtoname{$curfolder};
	$win = new Gtk2::Dialog;
	$win->set_title(_("Rename Folder ") ."$foldername");
	$win->signal_connect("destroy" => sub  { $win->destroy });
#	$win->signal_connect("delete-event" => sub{ Gtk2->false });
	$win->set_default_size('300','90');
#	$win->set_policy(1,1,0);
	$win->set_position(-mouse);
	$hbox = new Gtk2::HBox(0,5);
	$win->vbox->pack_start($hbox,1,1,0);
	$hbox->show;
	$win->show;
	$lbl = new Gtk2::Label(_("Rename ") ."\"$foldername\" to: ");
	$lbl->set_alignment(0.0,0.5);
	$lbl->show;
	$hbox->pack_start($lbl,0,0,5);
	$entry = new Gtk2::Entry;
	$entry->set_editable('1');  
	$entry->show;
	$hbox->pack_start($entry,1,0,1);
	$ok = Gtk2::Button->new_from_stock('gtk-ok');
	$ok->can_default(0);
	$ok->signal_connect("clicked", sub { 
					$newfoldername = $entry->get_text;
					if ($newfoldername eq '') { 
						&main::err_dialog(_("enter a name please!"));
						return 1; 
					} else {
						$sql = "update boxlist set name = '$newfoldername' where id ='$curfolder'";
						$query = $main::conn->prepare($sql);
						$query->execute();
						$win->destroy;
						&refresh_folder_name($curfolder, $newfoldername, $foldername);
					}
				});
	$ok->show;
	$win->action_area->pack_start($ok,1,1,0);
	$cancel = Gtk2::Button->new_from_stock('gtk-cancel');
	$cancel->signal_connect("clicked", sub { $win->destroy; });
	$cancel->can_default(1);
	$win->action_area->pack_start($cancel, 1, 1, 0);
	$cancel->grab_default;
	$cancel->show;
	return 1;
}

sub refresh_folder_name {

 	my ($folderid, $newfoldername, $oldname) = @_;
# 	$main::folder_tree->freeze;
	set_folder_name($main::folder_tree->get_model(), $main::ft_node[$folderid], $newfoldername);
# 	$main::folder_tree->thaw;
 	if ($folderid < 1000) {
		# $cp_menu_items{$folderid}->destroy;
 		$Pronto::FolderMenus::mv_menu_items{$folderid}->destroy;
		# $pcp_items{$folderid}->destroy;
 		$Pronto::FolderMenus::pmv_items{$folderid}->destroy; 
       		delete $main::folderidtoname{$folderid};
		delete $main::foldernametoid{$oldname};
 		$main::folderidtoname{$folderid} = $newfoldername;
 		$main::foldernametoid{$newfoldername} = $folderid;
		if($parent{$folderid} < 1000) {
		        $Pronto::FolderMenus::mv_menu_menus{$parent{$folderid}}->detach;
			$Pronto::FolderMenus::pmv_menus{$parent{$folderid}}->detach;
			$Pronto::FolderMenus::cp_menu_menus{$parent{$folderid}}->detach;
        		$Pronto::FolderMenus::pcp_menus{$parent{$folderid}}->detach;	
			$Pronto::FolderMenus::mv_menu_menus{$parent{$folderid}}->destroy;
			$Pronto::FolderMenus::pmv_menus{$parent{$folderid}}->destroy;
			$Pronto::FolderMenus::cp_menu_menus{$parent{$folderid}}->destroy;
        		$Pronto::FolderMenus::pcp_menus{$parent{$folderid}}->destroy;
			($Pronto::FolderMenus::mv_menu_menus{$parent{$folderid}},$Pronto::FolderMenus::pmv_menus{$parent{$folderid}},$Pronto::FolderMenus::cp_menu_menus{$parent{$folderid}},$Pronto::FolderMenus::pcp_menus{$parent{$folderid}})= &Pronto::FolderMenus::mvcp_menu($parent{$folderid});
			$Pronto::FolderMenus::mv_menu_items{$parent{$folderid}}->set_submenu($Pronto::FolderMenus::mv_menu_menus{$parent{$folderid}});
			$Pronto::FolderMenus::pmv_items{$parent{$folderid}}->set_submenu($Pronto::FolderMenus::pmv_menus{$parent{$folderid
			}});
			$Pronto::FolderMenus::cp_menu_items{$parent{$folderid}}->set_submenu($Pronto::FolderMenus::cp_menu_menus{$parent{$folderid}});
        		$Pronto::FolderMenus::pcp_items{$parent{$folderid}}->set_submenu($Pronto::FolderMenus::pcp_menus{$parent{$folderid
			}});
 		}
 	}
	&main::set_win_title;

return 1;
}
	      
sub delete_folder {
	my ($widget, $curfolder) = @_;
	my (@foldersel, $sql, $query);
	if (!$curfolder) {
		$curfolder = get_folder_id();
		if (!$curfolder) { return }
		
	}
	if ($curfolder <= 4) { return; }
	$main::folder_tree->{'no_dragging'} = 1;
	if ($parent{$curfolder} == 4) {
		$main::folder_tree->get_model->remove($main::ft_node[$curfolder]);
		$sql = "delete from boxlist where id=$curfolder";
		$main::conn->do($sql);
		$sql = "delete from filters where boxid=$curfolder and type != 6";
		$main::conn->do($sql);
		$sql = "select id from messages where boxid=?";
		$query = $main::conn->prepare($sql);
		$query->execute($curfolder);
		while (my ($msgid)=$query->fetchrow_array()) {
			&Pronto::Data::Message::delete($msgid);
		}	
		if ($curfolder < 1000) {
			$Pronto::FolderMenus::mv_menu_menus{0}->detach;
        		$Pronto::FolderMenus::pmv_menus{0}->detach;
			$Pronto::FolderMenus::cp_menu_menus{0}->detach;
                        $Pronto::FolderMenus::pcp_menus{0}->detach;
        		$Pronto::FolderMenus::mv_menu_menus{0}->destroy;
        		$Pronto::FolderMenus::pmv_menus{0}->destroy;
			$Pronto::FolderMenus::cp_menu_menus{0}->destroy;
                	$Pronto::FolderMenus::pcp_menus{0}->destroy;
			($Pronto::FolderMenus::mv_menu_menus{0},$Pronto::FolderMenus::pmv_menus{0},$Pronto::FolderMenus::cp_menu_menus{0},$Pronto::FolderMenus::pcp_menus{0})= &Pronto::FolderMenus::mvcp_menu(0);
        		$Pronto::FolderMenus::mv_menu_items{0}->set_submenu($Pronto::FolderMenus::mv_menu_menus{0});
        		$Pronto::FolderMenus::pmv_items{0}->set_submenu($Pronto::FolderMenus::pmv_menus{0});
			$Pronto::FolderMenus::cp_menu_items{0}->set_submenu($Pronto::FolderMenus::cp_menu_menus{0});
                	$Pronto::FolderMenus::pcp_items{0}->set_submenu($Pronto::FolderMenus::pcp_menus{0});
		}
		delete $main::foldernametoid{$main::folderidtoname{$curfolder}};
		delete $main::folderidtoname{$curfolder};
	} else {
		my $iter = $main::folder_tree->get_model->append($main::ft_node[4]);

		# It's a pity, that move_after doesn't work in this case :-(
		$main::folder_tree->get_model->move_after($main::ft_node[$curfolder], $iter);

		for (my $i=0; $i<5; $i++) {
			my $val = $main::folder_tree->get_model->get($main::ft_node[$curfolder], $i);
			$main::folder_tree->get_model->set($iter, $i, $val);
		}
		$main::folder_tree->get_model->remove($main::ft_node[$curfolder]);
		$main::ft_node[$curfolder] = $iter;
		
		$main::folder_tree->expand_all;
		$parent{$curfolder} = 4;
		$sql = "select id from boxlist where sibling=$curfolder";
		$query = $main::conn->prepare($sql);
		$query->execute();
		if ($query->rows() == 1) {
			# previous sibling
			my ($prev_sibling) = $query->fetchrow();

			$sql = "select sibling from boxlist where id=$curfolder";
			$query = $main::conn->prepare($sql);
			$query->execute();
			if ($query->rows() > 1) {
				# big error! somehow our id doesn't exist
				return 0;
			}
			my ($next_sibling) = $query->fetchrow();

			$sql = "update boxlist set sibling=$next_sibling where id=$prev_sibling";
			$main::conn->do($sql);
		} elsif ($query->rows() > 1) {
			# somehow we have multiple siblings pointing to us = bad
			return 0;
		}

		$sql = "select id from boxlist where parent=4 and sibling=65535";
		$query = $main::conn->prepare($sql);
		$query->execute();
		if (my ($deleted_sibling) = $query->fetchrow()) {
			$sql = "update boxlist set sibling=$curfolder where id=$deleted_sibling";
			$main::conn->do($sql);
		}
		$sql = "update boxlist set parent=4,sibling=65535 where id=$curfolder";
		$main::conn->do($sql);
	}
	$main::folder_tree->{'no_dragging'} = 0;
	return 1;
}

sub add_new_folder {
	
	my $widget = shift;
	my ($entry, $win, $fldr_ctree, $virtual, $querycontent, $body) = @{shift(@_)};
	my ($nfparent, $foldername, $sel, $model);
	$foldername = $entry->get_text;
	if ($foldername eq '') { &main::err_dialog(_("Enter a name first !")); return 1; }
	foreach (keys(%main::foldernametoid)) {
		if (lc($foldername) eq lc($_)) {
			&main::err_dialog(_("Unique names please!"));
			return 1;
		}
	}
	$main::folder_tree->{'no_dragging'} = 1;

 	($model, $sel) = $fldr_ctree->get_selection->get_selected;
 	if ($sel) {
  		$nfparent = $main::foldernametoid{get_folder_name($model, $sel)};
		if (!$nfparent) { 
			print(_("couldn't get parent for new folder, creating under the root\n"));
			$nfparent = 0;
	       	}
 	} else {
  		$nfparent = 0;
 	}
	
 	$win->destroy;	
	my ($sql, $query, $sort, $id);
 	$sql="select id from boxlist where ((parent = ?) and (sibling = 65535))";
 	$query = $main::conn->prepare($sql);
 	$query->execute($nfparent);
 	my ($oldtail) = $query->fetchrow_array();
 	if (not defined $virtual) {
 		$sql="insert into boxlist (id, name, parent, sibling) values (?,?,?,?)";
  		$query = $main::conn->prepare($sql);
 		$id = &main::newid_folder('boxlist', $main::conn);
 		$query->execute($id, $foldername, $nfparent, 65535);
 	} else {
 		$sql = "insert into boxlist (id,name,parent,sibling,query,body) values (?,?,?,?,?,?)";
 		$query = $main::conn->prepare($sql);
 		$id = &main::newid1000('boxlist',$main::conn);
 		$query->execute($id, $foldername, $nfparent, 65535, $querycontent, $body);
 	}
 	if (defined $oldtail) {
  		$sql="update boxlist set sibling = ? where id = ?";
  		$query = $main::conn->prepare($sql);
  		$query->execute($id,$oldtail);
 	}

 	$main::foldernametoid{$foldername} = $id;
 	$main::folderidtoname{$id} = $foldername;
	my $iter = $model->append($main::ft_node[$nfparent]);
	set_folder_name($model, $iter, $foldername);
	$model->set($iter, MSG_COL, 0, NEWMSG_COL, 0, BOLD_COL, FALSE);
	$main::folder_tree->expand_all;
	$main::ft_node[$id] = $iter;
 	$parent{$id}=$nfparent;
	$main::folder_tree->{'no_dragging'} = 0;

 	if (not defined $virtual) {
  		if(defined $Pronto::FolderMenus::mv_menu_menus{$parent{$id}}) {
			$Pronto::FolderMenus::mv_menu_menus{$parent{$id}}->detach;
  			$Pronto::FolderMenus::pmv_menus{$parent{$id}}->detach;
			$Pronto::FolderMenus::cp_menu_menus{$parent{$id}}->detach;
        		$Pronto::FolderMenus::pcp_menus{$parent{$id}}->detach;
  			$Pronto::FolderMenus::mv_menu_menus{$parent{$id}}->destroy;
  			$Pronto::FolderMenus::pmv_menus{$parent{$id}}->destroy;
			$Pronto::FolderMenus::cp_menu_menus{$parent{$id}}->destroy;
        		$Pronto::FolderMenus::pcp_menus{$parent{$id}}->destroy;
  		}

		($Pronto::FolderMenus::mv_menu_menus{$parent{$id}},$Pronto::FolderMenus::pmv_menus{$parent{$id}},$Pronto::FolderMenus::cp_menu_menus{$parent{$id}},$Pronto::FolderMenus::pcp_menus{$parent{$id}})= &Pronto::FolderMenus::mvcp_menu($parent{$id});
  		$Pronto::FolderMenus::mv_menu_items{$parent{$id}}->set_submenu($Pronto::FolderMenus::mv_menu_menus{$parent{$id}});
  		$Pronto::FolderMenus::pmv_items{$parent{$id}}->set_submenu($Pronto::FolderMenus::pmv_menus{$parent{$id}});
  		$Pronto::FolderMenus::cp_menu_items{$parent{$id}}->set_submenu($Pronto::FolderMenus::cp_menu_menus{$parent{$id}});
  		$Pronto::FolderMenus::pcp_items{$parent{$id}}->set_submenu($Pronto::FolderMenus::pcp_menus{$parent{$id}});
 	}
	&set_folder_style;
 	&refresh_folder_tree;
 
 	return 1;
}


sub init_export_folder {
 
 	my ($widget, $listwidget)=@_;
 	my ($fs_window, $sql, $query, $curfolder, $foldername);
        $curfolder = get_folder_id();	
	if (!$curfolder) {
		&main::err_dialog(_("Please select a folder first!")); 
		return 1; 
       }
 
 	$sql = "select name from boxlist where id = '$curfolder'";
 	$query=$main::conn->prepare($sql);
 	$query->execute();
 	$foldername = $query->fetchrow_array();
 	$fs_window = new Gtk2::FileSelection (_("Choose Directory to export folder ") ."($foldername) to  ...");
 	$fs_window->set_position(-mouse);
 	$fs_window->file_list->set_sensitive(0);
 
 	$fs_window->signal_connect("destroy", sub {$fs_window->destroy;});
#   	$fs_window->signal_connect("delete-event" => sub { Gtk2->false } );
    	$fs_window->ok_button->signal_connect("clicked", \&main::export_folder, [$fs_window, $listwidget, $main::conn, $curfolder, $foldername]);
    	$fs_window->cancel_button->signal_connect("clicked", sub {$fs_window->destroy;});
     	$fs_window->show;
      	return 1;
}
      
sub init_import_folder {

 	my ($widget, $listwidget)=@_;
 	my ($fs_window, $sql, $query, $curfolder, $foldername);
  	$curfolder = get_folder_id();
	if (!$curfolder) {
	 	&main::err_dialog(_("Please select a folder first!"));
		return 1; 
	}
 
  	$sql = "select name from boxlist where id = '$curfolder'";		     
  	$query=$main::conn->prepare($sql);
  	$query->execute();
  	$foldername = $query->fetchrow_array();
 	$fs_window = new Gtk2::FileSelection (_("Choose File to import into folder ") . "($foldername)  ...");
 	$fs_window->set_position(-mouse);
 	$fs_window->signal_connect("destroy", sub {$fs_window->destroy;});
# 	$fs_window->signal_connect("delete-event" => sub { Gtk2->false } );
 	$fs_window->ok_button->signal_connect("clicked", \&main::import_folder, [$fs_window, $listwidget, $main::conn, $curfolder, $foldername]);
 	$fs_window->cancel_button->signal_connect("clicked", sub {$fs_window->destroy;});
 	$fs_window->show;
      	return 1;
}

sub refilter_folder 
{
	&Pronto::Data::Message::refilter("folder");
	return 1;
}	

sub empty_folder {

     	my ($sql, $sql2, $query,$curfolder, @row);
     
	$curfolder = get_folder_id();
	if (!$curfolder) { &main::err_dialog(_("Please select a folder first!")); return 1; }
	if ($curfolder == 4) { &main::err_dialog(_("Use Empty deleted items for the trash folder")); return 1;}

	$sql = "select id from messages where boxid = '$curfolder'";
	$query=$main::conn->prepare($sql);
	$query->execute();
	while (@row = $query->fetchrow_array()) {
		$sql = "update messages set boxid = 4  where id = @row";
		$main::conn->do($sql);
		if ($main::prefs{'DeleteMarksAsRead'} eq "y") { 
			$sql2 = "update messages set newmsg = 'n' where id = @row";
			$main::conn->do($sql2);
		}
		
		if ($main::prefs{'EmptyNoBlock'}) {
			while (Gtk2->events_pending()) {
				Gtk2->main_iteration();
			}	
		}	
	}
	&refresh_folder_tree;
	$main::message_clist->get_model->clear;
	if (get_folder_id() == $curfolder) {
		&Pronto::Read::clear_widget();
	}	
	return 1;
}

sub set_folder_style {
	my $pixbuf;
	foreach(keys(%main::folderidtoname)) {	
		if ($_ < 1000) {
			$pixbuf = $main::folder_tree->{'pix'}->{'dummy'};
		} elsif ($_ == 1000) {
			$pixbuf = $main::folder_tree->{'pix'}->{'search'};
		} else {
			$pixbuf = $main::folder_tree->{'pix'}->{'vfolder'};
		}

		$main::folder_tree->get_model->set($main::ft_node[$_], ICON_COL, $pixbuf);
	}
	return 1;
}

sub empty_trash {
	my ($widget,$at_exit) = @_;
	my ($sql, $query, @foldersel, $curfolder,@row);
	$sql = "select id from boxlist where parent = 4";
	$query=$main::conn->prepare($sql);
	$query->execute();
	while (@row=$query->fetchrow_array()) {
		&delete_folder(undef, $row[0]);
	}
	$sql = "select id from messages where boxid = 4";
	$query=$main::conn->prepare($sql);
	$query->execute();
	while (my ($msgid)=$query->fetchrow_array()) {
		&Pronto::Data::Message::delete($msgid);
		if ($main::prefs{'EmptyNoBlock'}) {
			while (Gtk2->events_pending()) {
				Gtk2->main_iteration();
			}	
		}
	}
	$curfolder = &get_folder_id();
	if ($curfolder) {
		if ($curfolder == 4) { 
			$main::message_clist->get_model->clear;
			&Pronto::Read::clear_widget();
		}
	}
	if (!$at_exit) {
		&Pronto::FolderTree::refresh_folder_tree; 
	}
	return 1;
}




sub rightclick 
{
	my ($widget, $event) = @_;
	if (($event->type() eq 'button-press') && ($event->button() == 3) && ($widget->{'popup'})) {
   		$widget->{'popup'}->popup(undef,undef,undef,1,$event->button,$event->time);
	}
     
     return FALSE;
}

sub mark_folder_as_read
{
	my $folder = get_folder_id();
	if (!$folder) { return 1; }
	my $sql = "update messages set newmsg = 'n' where boxid = $folder";
	$main::conn->do($sql);
	&Pronto::MessageList::refresh();
	&refresh_folder_tree;
	return 1;
}	

sub goto_next_unread_folder {
	my ($model, $iter);

	$model = $main::folder_tree->get_model;
	$iter = $main::folder_tree->get_selection->get_selected;
	$iter = $model->iter_children($model->get_iter_first) if ($_[0] && $_[0] eq "top");

	$iter = &Pronto::MessageList::walk_through_store(\&has_folder_new_messages, $model, $iter);

	if ($iter) {
		$main::folder_tree->get_selection->select_iter($iter);
		$main::folder_tree->scroll_to_cell($model->get_path($iter), undef, 1, 0.5, 0.5);
		return 1;
	}
	
	return 0;
}

sub has_folder_new_messages {
	my ($store, $iter) = @_;

	return 1 if ($store->get($iter, NEWMSG_COL));
	return 0;
}

1;
