# $Id: KeyEdit.pm,v 1.6 2002/05/16 12:00:50 muhri Exp $
# -*- perl -*-
package Pronto::KeyEdit;
use strict;
use vars qw ( %gdk_key %realkey );

sub get_hash
{

	my @gdk_keys = &get_array("gdk_keys");
	my $i = 0;
	foreach(&get_array("keys")) {
		$gdk_key{$_} = $gdk_keys[$i];
		$realkey{$gdk_keys[$i]} = $_;
		$i++;
	}
	my ($alt,$ctl,$none) = &get_array("mods");
	$gdk_key{'Empty-Folder'} = 0x064;		
	$gdk_key{'Empty-Folder'.'mod'} = $ctl;	
	$gdk_key{'Mini-Browser'} = 0x062;
	$gdk_key{'Mini-Browser'.'mod'} = $alt;
	$gdk_key{'Save'} = 0x073;
	$gdk_key{'Save'.'mod'} = $alt;
	$gdk_key{'Exit'} = 0x078;
	$gdk_key{'Exit'.'mod'} = $alt;
	$gdk_key{'Send-and-Receive'} = 0x06d;
	$gdk_key{'Send-and-Receive'.'mod'} = $alt;
	$gdk_key{'AdressBook'} = 0x061;
	$gdk_key{'AdressBook'.'mod'} = $alt;
	$gdk_key{'New-Message'} = 0x06e;
	$gdk_key{'New-Message'.'mod'} = $alt;
	$gdk_key{'Options'} = 0x06f;
	$gdk_key{'Options'.'mod'} = $ctl;
	$gdk_key{'Reply'} = 0x072;
	$gdk_key{'Reply'.'mod'} = $alt;
	$gdk_key{'Reply-To-List'} = 0x06c;
	$gdk_key{'Reply-To-List'.'mod'} = $alt;
	$gdk_key{'Forward'} = 0x066;
	$gdk_key{'Forward'.'mod'} = $alt;
	$gdk_key{'View-Win'} = 0x076;
	$gdk_key{'View-Win'.'mod'} = $ctl;
	$gdk_key{'View-Header'} = 0x068;
	$gdk_key{'View-Header'.'mod'} = $ctl;
	$gdk_key{'View-Source'} = 0x073;
	$gdk_key{'View-Source'.'mod'} = $ctl;
	$gdk_key{'Delete'} = 0x064;
	$gdk_key{'Delete'.'mod'} = $alt;
	$gdk_key{'Select-All'} = 0x061;
	$gdk_key{'Select-All'.'mod'} = $ctl;
	$gdk_key{'Thread-Subject'} = 0x073;
	$gdk_key{'Thread-Subject'.'mod'} = $none;
	$gdk_key{'KeyEdit'} = 0x06b;
	$gdk_key{'KeyEdit'.'mod'} = $alt;
	$gdk_key{'NextUnread'} = 0x020;
	$gdk_key{'NextUnread'.'mod'} = $alt;
	$gdk_key{'Reply-All-Including-Self'} =  0x069;
	$gdk_key{'Reply-All-Including-Self'.'mod'} = $alt;
	$gdk_key{'Reply-All-Execluding-Self'} = 0x077;
	$gdk_key{'Reply-All-Execluding-Self'.'mod'} = $alt;
	$gdk_key{'Edit'} =  0x065;
	$gdk_key{'Edit'.'mod'} = $alt;
	$gdk_key{'Toggle-Threads'} = 0x074;
	$gdk_key{'Toggle-Threads'.'mod'} = $alt;
	$gdk_key{'HTML-View'} = 0x06d;
	$gdk_key{'HTML-View'.'mod'} = $ctl;
	$gdk_key{'Verify'} = 0x076;
	$gdk_key{'Verify' . 'mod'} = $alt;
	$gdk_key{'Decrypt'} = 0x06b;
	$gdk_key{'Decrypt' . 'mod'} = $ctl;

	read_keys();	
	return %gdk_key;
}

sub init_key_editor
{
	my ($dialog, $model, $renderer, $column, @titles, $colcount);
	$dialog = new Gtk2::Dialog;
	$dialog->set_title(_("Pronto's Key bindings Editor"));
	$dialog->set_default_size(300,300);
	$dialog->signal_connect("destroy", sub { $dialog->destroy; });

	$model = Gtk2::ListStore->new('Glib::String', 'Glib::String', 'Glib::String');
	$dialog->{'clist'} = Gtk2::TreeView->new($model);
	$dialog->{'clist'}->get_selection->set_mode("single");
	$dialog->{'clist'}->set_rules_hint(1);
	@titles = (_("Action"),_("Modifier"),_("Key"));
	$colcount = 0;
	foreach (@titles) {
		$renderer = Gtk2::CellRendererText->new;
		$column = Gtk2::TreeViewColumn->new_with_attributes($_, $renderer, text => $colcount);
		$column->set_sort_column_id($colcount);
		$column->set_resizable(1);
		$column->set_sizing('fixed');
		if ($colcount == 0) {
			$column->set_fixed_width(150);
		} else {
			$column->set_fixed_width(60);
			$column->set_alignment(0.5);
			$renderer->set('xalign', 0.5);
		}
		$dialog->{'clist'}->append_column($column);
		$colcount++;
	}

	$dialog->{'sw'} = new Gtk2::ScrolledWindow(undef,undef);
	$dialog->{'sw'}->set_policy('automatic','automatic');
	$dialog->{'sw'}->set_shadow_type('etched-in');
	$dialog->{'sw'}->add($dialog->{'clist'});
	$dialog->vbox->pack_start($dialog->{'sw'},1,1,0);
	$dialog->{'ok'} = Gtk2::Button->new_from_stock('gtk-ok');
	$dialog->{'ok'}->signal_connect("clicked" => \&ok, $dialog);
	$dialog->{'edit'} = new Gtk2::Button(_("Edit"));
	$dialog->{'edit'}->signal_connect("clicked" => \&edit, $dialog->{'clist'});
	$dialog->{'cancel'} = Gtk2::Button->new_from_stock('gtk-cancel');
	$dialog->{'cancel'}->signal_connect("clicked", sub { $dialog->destroy; });
	$dialog->action_area->pack_start($dialog->{'ok'},1,1,0);
	$dialog->action_area->pack_start($dialog->{'edit'},1,1,0);
	$dialog->action_area->pack_start($dialog->{'cancel'},1,1,0);
	insert_into_list($dialog->{'clist'});
	show_all $dialog;
}

sub insert_into_list
{
	my ($clist) = @_;
	my $model = $clist->get_model;
	my $iter;
	
	foreach(&get_array("commands")) {
		$iter = $model->append;
		$model->set($iter, 0, $_, 1, $gdk_key{$_."mod"}, 2, $realkey{$gdk_key{$_}});
	}	
}	

sub get_array
{
	my ($array) = @_;
	if (!$array) { return undef; }
	if ($array eq "commands") {
		my @commands = ("Mini-Browser","Save","Exit",
				"Select-All","Options","Send-and-Receive",
				"AdressBook","New-Message","Reply",
				"Reply-To-List","Edit","View-Win","View-Header",
				"View-Source", "Delete","Thread-Subject"
				,"Forward","KeyEdit","NextUnread","Reply-All-Including-Self",
				"Reply-All-Execluding-Self","Toggle-Threads",
				"HTML-View", "Empty-Folder","Verify","Decrypt");
		return @commands;
	}
	if ($array eq "mods") {
		my @mods = ("Alt","Ctrl","None");
		return @mods;
	}
	if ($array eq "keys") {		
		my @key = (
			   "a","b","c","d","e",
			   "f","g","h","i","j",
			   "k","l","m","n","o",
			   "p","q","r","s","t",
			   "u","v","w","x","y",
			   "z","space"
			   );
		return @key;
	}	
	if ($array eq "gdk_keys") {
		my @gdk_keys = (
				0x061,0x062,0x063,0x064,
				0x065,0x066,0x067,0x068,
				0x069,0x06a,0x06b,0x06c,
				0x06d,0x06e,0x06f,0x070,
				0x071,0x072,0x073,0x074,
				0x075,0x076,0x077,0x078,
				0x079,0x07a,0x020
       				);
		return @gdk_keys;
	}	
}

sub ok
{
	my ($button, $dialog) = @_;
	$dialog->destroy();
	
}

sub edit
{
	my ($button,$clist) = @_;
	my ($selection,$dialog,$action,$mod,$key);
	$selection = $clist->get_selection->get_selected;
	if (!$selection) { return 1; }
	$dialog = new Gtk2::Dialog;
	$dialog->set_title(_("Edit Key"));
	$action = $clist->get_model->get($selection, 0);
	$mod = $clist->get_model->get($selection, 1);
	$key = $clist->get_model->get($selection, 2);
	$dialog->{'action'} = new Gtk2::Entry;
	$dialog->{'action'}->set_editable(0);
	$dialog->{'mod'} = new Gtk2::Combo;
	$dialog->{'key'} = new Gtk2::Combo;
	$dialog->{'mod'}->set_popdown_strings(&get_array("mods"));
	$dialog->{'key'}->set_popdown_strings(&get_array("keys"));
	$dialog->{'action'}->set_text($action);
	$dialog->{'mod'}->entry->set_text($mod);
	$dialog->{'key'}->entry->set_text($key);
	$dialog->{'ok'} = Gtk2::Button->new_from_stock('gtk-ok');
	$dialog->{'ok'}->signal_connect("clicked" => \&process_entry, [$dialog, $clist, $selection]);
	$dialog->{'cancel'} = Gtk2::Button->new_from_stock('gtk-cancel');
	$dialog->{'cancel'}->signal_connect("clicked", sub { $dialog->destroy });
	$dialog->vbox->pack_start($dialog->{'action'},0,0,0);
	$dialog->vbox->pack_start($dialog->{'mod'},0,0,0);
	$dialog->vbox->pack_start($dialog->{'key'},0,0,0);
	$dialog->action_area->pack_start($dialog->{'ok'},1,1,0);
	$dialog->action_area->pack_start($dialog->{'cancel'},1,1,0);
	show_all $dialog; 
	return 1;
}	

sub process_entry
{
	my $button = shift;
	my ($dialog,$clist, $selection) = @{shift(@_)};
	my ($action,$mod,$key);
	$action = $dialog->{'action'}->get_text;
	$mod = $dialog->{'mod'}->entry->get_text;
	$key = $dialog->{'key'}->entry->get_text;
	$gdk_key{$action} = $gdk_key{$key};
	$gdk_key{$action.'mod'} = $mod;
	$dialog->destroy;
	$clist->get_model->set($selection, 1, $mod, 2, $key);
	&make_key_config();
	return 1;
}	

sub read_keys 
{
	if (!-f "$main::prefs{'MailDir'}/pronto.keys") {
		make_key_config();
	}	
	open(KEYS, "$main::prefs{'MailDir'}/pronto.keys");
	while (<KEYS>) {
		chomp;
		my @key = split(/,/,$_);
		$gdk_key{$key[0].'mod'} = $key[1];
		$gdk_key{$key[0]} = $key[2];
		
	}
	close(KEYS);
	make_key_config();
}

sub make_key_config
{
	open(TMP, ">$main::prefs{'MailDir'}/pronto.keys");
	foreach (&get_array("commands")) {
		print TMP $_ . "," . $gdk_key{$_.'mod'} . "," . $gdk_key{$_} . "\n";
	}
	close(TMP);
	chmod(0600, "$main::prefs{'MailDir'}/pronto.keys");

}

1;
